KEY_PRESS = 0

MOUSE_DOWN = 1
MOUSE_UP = 2
MOUSE_DOUBLE_CLICK = 3
MOUSE_MOVE = 5

SCROLL_DOWN = 6
SCROLL_UP = 7
SCROLL_STEP = 1

CTRL = 'ctrl'
SHIFT = 'shift'
ALT = 'alt'

MODIFIER_KEYS = (CTRL, SHIFT, ALT,)

MODIFIER_KEYS_PRESS_DELAY = .4
EVENTS_DELAY = .05

LEFT = "left"
MIDDLE = "middle"
RIGHT = "right"

QUALITY = 80

HIGH_SCALE = 1
LOW_SCALE = 2
